// filename: ATMCell.h
//
// Default values set for ATMCell to contain 53 bytes with
// 8 bits/byte or 424 bits. The variable "cell" will be a pointer
// to an array of (unsigned short)-types. The array will be large
// enough to contain designated number of bits. The ATMCell idea,
// as well as much of the following code, was taken from
// C++ PRIMER by Stanley B. Lippman, Addison Weslsey, 2nd ed., 1991.

#ifndef ATMCell_h
#define ATMCell_h
#include "Message.h"

class ATMCell : public Message {
public:
	~ATMCell() { LOG_DEL; delete [] cell; }

//	ATMCell& unset( const int );
	void setSeq( int num ) { seqNo = num; }


// returns TRUE if ith bit is OFF
int isOFF( const int ith ) const {
        return( !( this->isON( ith ) ) );
}

	int readSeq() const { return( seqNo ); }
        int readSource() const { return( source ); }
        int readDestination() const { return( destination ); }
        int readVC() const { return( VC); }
        void setSource( int num ) { source= num; }
        void setDestination( int num ) { destination= num; }
        void setVC( int num ) { VC= num; }

	float readACR() const{ return (ACR); }
	int   readDIR() const{ return (DIR); }
	float readER()  const{ return (ER);  }
	int   readCI()  const{ return (CI);  }
	int   readRM()  const{ return (RM);  }
	int   readEFCI()const{ return (EFCI); }

	void setACR( float num ) { ACR = num; }
	void setDIR( int num )   { DIR = num; }
	void setER( float num ) { ER = num; }
	void setCI( int num ) { CI = num; }
	void setRM( int num ) { RM = num; }
	void setEFCI( int num ) { EFCI = num; }

	Message* clone() const{
                LOG_NEW; return new ATMCell( *this );
        }

	double asFloat() const { return( double( seqNo ) ); }
	unsigned short* cell;

StringList print() const {
	StringList out;
	out << "\nATMCell: seqNo = ";
	out << readSeq() << "\n";
        out << "source= "<< readSource()<<"  destination= "
            << readDestination() << " VC= "<< readVC()<< '\n'; 
	
	return out;
}

// class identification
//ISA_FUNC(ATMCell,Message);

// 424 = 8 * 53, and there are 53 bytes in the proposed broadband
// integrated serviced digital network (BISDN) cell.
// Asynchronous transfer mode (ATM) is the switching method used
// by the BISDN.


// default constructor
ATMCell( const int sequenceNo ) : seqNo( sequenceNo ),
		size( 424 )
{
// 'wordWidth' is minimum number of unsigned short needed
// to hold 'size' bits.
	const int wordWidth = ( size + 8*sizeof(short) - 1 ) /
			8*sizeof(short);
	LOG_NEW;
	cell = new unsigned short[ wordWidth ];
	for ( int i = 0; i < wordWidth; ++i ) // initialize all bits to 0
		cell[ i ] = 0;
}

// copy constructor
ATMCell( const ATMCell& ba ) : size( 424 )
{
	const int wordWidth = ( size + 8*sizeof(short) - 1 ) /
			8*sizeof(short);
	LOG_NEW;
	cell = new unsigned short[ wordWidth ];
	for ( int i = 0; i < wordWidth; ++i )
		cell[ i ] = ba.cell[ i ];
	this->seqNo = ba.seqNo;
}

// returns TRUE if ith bit is ON
int isON( const int ith ) const {
        int index = ith / 8*sizeof(short);
        int offset = ith % 8*sizeof(short);
        return( cell[ index ] & ( 1 << offset ) );
}
// turns on the ith bit
ATMCell& set( const int ith ) {
	int index = ith / 8*sizeof(short);
	int offset = ith % 8*sizeof(short);
	cell[ index ] = cell[ index ] | ( 1 << offset );
	return *this;
}

// turns off the ith bit
ATMCell& unset( const int ith ) {
	int index = ith / 8*sizeof(short);
	int offset = ith % 8*sizeof(short);
	cell[ index ] = cell[ index ] | ~( 1 << offset );
	return *this;
}

const char* dataType() const { return "ATMCell"; }

private:

	int seqNo;
	const int size; // number of bits desired in ATMCell
       
        // the following two fields are included to make 
        // routing and simulation debugging easier

        int source;     // Number of source node
        int destination;// Number of destination node
        int VC;// Number of VC node

	float ACR;  //Available Cell Rate
	float ER;  //Explicit Rate
	int DIR;   //Direction of the RM Cell
	int CI;  // Congestion Indicator;
	int RM;  // Resource Management;
	int EFCI;  //Explicit Forward Congestion Identification 


};


#endif
